
// *********************************************************************
// *  This software uses the Perseus API which is defined through the  *
// *  following four files of the Microtelecom Software Defined Radio  *
// *  Developer Kit (SDRK):                                            *
// *  main.cpp, perseusdll.cpp, perseusdefs.h, perseusdll.h            *
// *  To use the Perseus hardware you need the perseususb.dll file     *
// *  You also need at least one of the following files:               *
// *  perseus125k24v21.sbs                                             *
// *  perseus250k24v21.sbs                                             *
// *  perseus500k24v21.sbs                                             *
// *  perseus1m24v21.sbs                                               *
// *  perseus2m24v21.sbs                                               *
// *  The dll and the sbs files are copyright protected and only       *
// *  available at the official Microtelecom Internet site:            *
// *  http://www.microtelecom.it                                       *
// *********************************************************************

#include <stdio.h>
#include "wdef.h"
#include "globdef.h"
#include "uidef.h"
#include "fft1def.h"
#include "sdrdef.h"
#include "perseusdef.h"
#include "sdr14_dlldef.h"
#include "screendef.h"

extern P_PERSEUS pers;
extern char *perseus_bitstream_names[MAX_PERSEUS_RATES];


sdrWrite P_Write;
sdrRead P_Read;
sdrClose P_Close;
sdrSetTimeouts P_SetTimeouts;
FT_HANDLE sdr14_handle;

HINSTANCE h_sdr14;
HINSTANCE h_pers;

PtrToGetDLLVersion		m_pGetDLLVersion;
PtrToOpen		    	m_pOpen;
PtrToClose			    m_pClose;
PtrToGetDeviceName		m_pGetDeviceName;
PtrToGetDeviceName		m_pGetFriendlyDeviceName;
PtrToEepromFunc			m_pEepromRead;
PtrToFwDownload			m_pFwDownload;
PtrToFpgaConfig			m_pFpgaConfig;
PtrToSetAttPresel		m_pSetAttenuator;
PtrToSetAttPresel		m_pSetPreselector;
PtrToSio                m_pSio;
PtrToStartAsyncInput	m_pStartAsyncInput;
PtrToStopAsyncInput     m_pStopAsyncInput;

int timf1p_pers;
int perseus_bytesperbuf;
SIOCTL sioctl;
extern float perseus_filter_cutoff[MAX_PERSEUS_FILTER+1];


Key48 perseus48k24v31_signature= {{0xE4, 0x37, 0x73, 0x7B, 0x9A, 0x6A }};
Key48 perseus95k24v31_signature= {{0x22, 0x33, 0xB8, 0xEA, 0x2C, 0xBC }};
Key48 perseus96k24v31_signature= {{0x39, 0x18, 0x79, 0xF5, 0x90, 0x8C }};
Key48 perseus125k24v21_signature={{0x45, 0x7C, 0xD4, 0x3A, 0x0E, 0xD4 }};
Key48 perseus192k24v31_signature={{0x7B, 0x39, 0x26, 0x75, 0x9F, 0x7B }};
Key48 perseus250k24v21_signature={{0xCF, 0x4B, 0x9F, 0x02, 0xB2, 0x1E }};
Key48 perseus500k24v21_signature={{0xF2, 0x9E, 0x67, 0x49, 0x51, 0x6B }};
Key48 perseus1m24v21_signature=  {{0xC5, 0xC1, 0x96, 0x4D, 0xA8, 0x05 }};
Key48 perseus2m24v21_signature=  {{0xDE, 0xEA, 0x57, 0x52, 0x14, 0x35 }};


#if(OSNUM == OS_FLAG_WINDOWS)
Key48 *perseus_bitstream_signatures[MAX_PERSEUS_RATES]={
                               &perseus48k24v31_signature,
                               &perseus95k24v31_signature,
                               &perseus96k24v31_signature,
                               &perseus125k24v21_signature,
                               &perseus192k24v31_signature,
                               &perseus250k24v21_signature,
                               &perseus500k24v21_signature,
                               &perseus1m24v21_signature,
                               &perseus2m24v21_signature};
#endif





int WINAPI ProcessInputCallback(char *pBuf, int nothing)
{
register char c1;
int i, j;
c1=0;
// This callback gets called whenever a buffer is ready
// from the USB interface
j=timf1p_pers;
for(i=0; i<perseus_bytesperbuf; i+=6)
  {
  timf1_char[timf1p_pers+3]=pBuf[i+2];
  c1=pBuf[i+2]&0x80;
  timf1_char[timf1p_pers+2]=pBuf[i+1];
  timf1_char[timf1p_pers+1]=pBuf[i  ];
  c1|=pBuf[i];
  timf1_char[timf1p_pers  ]=0;
  timf1_char[timf1p_pers+7]=pBuf[i+5];
  timf1_char[timf1p_pers+6]=pBuf[i+4];
  timf1_char[timf1p_pers+5]=pBuf[i+3];
  timf1_char[timf1p_pers+4]=0;
  timf1p_pers=(timf1p_pers+8)&timf1_bytemask;
  }
if( (c1&1) != 0)
  {
  ad_maxamp[0]=0x7fff;
  ad_maxamp[1]=0x7fff;
  }
SetEvent(rxin1_bufready);
return nothing;
}

void lir_perseus_read(void)
{
if(((timf1p_pers-timf1p_pa+timf1_bytes)&timf1_bytemask) < (int)snd[RXAD].block_bytes)
  {
  WaitForSingleObject(rxin1_bufready, INFINITE);
  }
timf1p_pa=(timf1p_pa+snd[RXAD].block_bytes)&timf1_bytemask;
}

int perseus_load_proprietary_code(int rate_no)
{
int i;
// Download the standard Microtelecom FX2 firmware to the Perseus receiver
// IMPORTANT NOTE:
// The parameter passed to PerseusFirmwareDownload() must be NULL
// for the Perseus hardware to be operated safely.
// Permanent damages to the receiver can result if a third-party firmware
// is downloaded to the Perseus FX2 controller. You're advised.
//
// During the execution of the PerseusFirmwareDownload() function
// the Perseus USB hardware is renumerated. This causes the function
// to be blocking for a while. It is suggested to put an hourglass cursor
// before calling this function if the code has to be developed for a real
// Window application (and removing it when it returns)
i=(*m_pFwDownload)(NULL);
if(i != IHEX_DOWNLOAD_OK)
  {
  switch (i)
    {
    case IHEX_DOWNLOAD_IOERROR:
    return 1083;

    case IHEX_DOWNLOAD_FILENOTFOUND:
    return 1084;

    case IHEX_DOWNLOAD_INVALIDHEXREC:
    return 1085;

    case IHEX_DOWNLOAD_INVALIDEXTREC:
    return 1089;

    default:
    return 1090;
    }
  }
// Download the FPGA bitstream to the Perseus' FPGA
// There are three bitstream provided with the current software version
// one for each of the output sample rate provided by the receiver
// (125, 250, 500, 1000 or 2000 kS/s)
// Each FPGA bistream (.sbs file) has an original Microtelecom
// signature which must be provided as the second parameter to the
// function PerseusFpgaConfig() to prevent the configuration of the
// FPGA with non authorized, third-party, code which could damage
// the receiver hardware.
// The bitstream files should be located in the same directory of
// the executing software for things to work.
i=(*m_pFpgaConfig)( perseus_bitstream_names[rate_no],
                    perseus_bitstream_signatures[rate_no]);
if(i != FPGA_CONFIG_OK)
  {
  switch (i)
    {
    case FPGA_CONFIG_FILENOTFOUND:
    return 1092;

    case FPGA_CONFIG_IOERROR:
    return 1096;

    default:
    return 1097;
    }
  }
return 0;
}

int perseus_eeprom_read(unsigned char *buf, int addr, unsigned char count)
{
return (*m_pEepromRead)(buf, addr, count);
}

char *perseus_name(void)
{
return (*m_pGetDeviceName)();
}

char *perseus_frname(void)
{
return (*m_pGetFriendlyDeviceName)();
}

void perseus_store_att(unsigned char idAtt)
{
m_pSetAttenuator(idAtt);
}



void perseus_store_presel(unsigned char idPresel)
{
m_pSetPreselector(idPresel);
}

void set_perseus_frequency(void)
{
unsigned char sl;
double dt1;
sl=PERSEUS_FLT_WB;
if(pers.presel !=0 )
  {
  sl=0;
// select filter according to frequency.
  while(perseus_filter_cutoff[sl]<fg.passband_center)sl++;
  }
perseus_store_presel(sl);
dt1=fg.passband_center*1000000+0.5;
if(dt1 < 0 || dt1 > adjusted_sdr_clock/2)
  {
  fg.passband_center=7.05;
  dt1=fg.passband_center*1000000;
  }
sioctl.freg=4.294967296E9*dt1/adjusted_sdr_clock;
m_pSio(&sioctl,sizeof(sioctl));
}




void open_perseus(void)
{
int j, k;
if(sdr != -1)return;
h_pers = LoadLibrary("perseususb.dll");
if(h_pers == NULL)
  {
  SNDLOG"\nUnable to load perseususb.dll");
  return;
  }
if(write_log)
  {
  lir_text(10,13,
         "If Linrad makes a silent exit, try again. If it does not work,");
  lir_text(10,14,
         "remove perseususb.dll if you do not want to use the Perseus.");
  lir_refresh_screen();
  lir_sleep(1500000);
  clear_lines(13,14);
  }
m_pGetDLLVersion	= (PtrToGetDLLVersion)GetProcAddress(h_pers,"GetDLLVersion");
m_pOpen                 = (PtrToOpen)GetProcAddress(h_pers,"Open");
m_pClose                = (PtrToClose)GetProcAddress(h_pers,"Close");
m_pGetDeviceName	= (PtrToGetDeviceName)GetProcAddress(h_pers,"GetDeviceName");
m_pGetFriendlyDeviceName= (PtrToGetDeviceName)GetProcAddress(h_pers,"GetFriendlyDeviceName");
m_pEepromRead		= (PtrToEepromFunc)GetProcAddress(h_pers,"EepromRead");
m_pFwDownload           = (PtrToFwDownload)GetProcAddress(h_pers,"FirmwareDownload");
m_pFpgaConfig		= (PtrToFpgaConfig)GetProcAddress(h_pers,"FpgaConfig");
m_pSetAttenuator	= (PtrToSetAttPresel)GetProcAddress(h_pers,"SetAttenuator");
m_pSetPreselector	= (PtrToSetAttPresel)GetProcAddress(h_pers,"SetPreselector");
m_pSio			= (PtrToSio)GetProcAddress(h_pers,"Sio");
m_pStartAsyncInput	= (PtrToStartAsyncInput)GetProcAddress(h_pers,"StartAsyncInput");
m_pStopAsyncInput	= (PtrToStopAsyncInput)GetProcAddress(h_pers,"StopAsyncInput");
SNDLOG"\nperseususb.dll loaded");
if( m_pGetDLLVersion == NULL ||
    m_pOpen == NULL ||
    m_pGetDeviceName == NULL ||
    m_pGetFriendlyDeviceName == NULL ||
    m_pEepromRead == NULL ||
    m_pFwDownload == NULL ||
    m_pFpgaConfig == NULL ||
    m_pSetAttenuator == NULL ||
    m_pSetPreselector == NULL ||
    m_pSio  == NULL ||
    m_pStartAsyncInput == NULL ||
    m_pStopAsyncInput == NULL
    )
  {
  SNDLOG"\nDLL function(s) missing");
free_pdll:;
  FreeLibrary(h_pers);
  return;
  }
j = m_pGetDLLVersion();
if (!j)
  {
  SNDLOG"\nDLLversion failed");
  goto free_pdll;
  }
k=j>>16;
j&=0xffff;
SNDLOG" vers %d.%d",k,j);
j=m_pOpen();
if(j==FALSE)
  {
  SNDLOG"\nPerseus receiver not detected");
  if(write_log)fflush(sndlog);
  goto free_pdll;
  }
sioctl.ctl=0;
if(pers.preamp != 0)sioctl.ctl|=PERSEUS_SIO_GAINHIGH;
if(pers.dither != 0)sioctl.ctl|=PERSEUS_SIO_DITHER;
sdr=0;
}

void start_perseus_read(void)
{
int i;
sdr=-1;
rxin1_bufready=CreateEvent( NULL, FALSE, FALSE, NULL);
if(rxin1_bufready== NULL)
  {
  close_perseus();
  lirerr(1223);
  return;
  }
perseus_bytesperbuf=snd[RXAD].block_bytes/PERSEUS_BYTESPERFRAME;
perseus_bytesperbuf*=PERSEUS_BYTESPERFRAME;
timf1p_pers=timf1p_pa;
lir_sched_yield();
i=m_pStartAsyncInput(perseus_bytesperbuf, ProcessInputCallback, 0);
if(i == FALSE)
  {
  close_perseus();
  lirerr(1098);
  return;
  }
// Enable the Perseus data FIFO through its SIO interface
// setting the PERSEUS_SIO_FIFOEN bit in the ctl field of
// the (global) sioctl variable.
sioctl.ctl |=PERSEUS_SIO_FIFOEN;
m_pSio(&sioctl,sizeof(sioctl));
sdr=0;
}

void perseus_stop(void)
{
m_pStopAsyncInput();
sioctl.ctl &=~PERSEUS_SIO_FIFOEN;
m_pSio(&sioctl,sizeof(sioctl));
lir_sleep(10000);
}

void close_perseus(void)
{
int i;
//return;  //  There is a bug that prevents close and re-open.
i=m_pClose();
i=FreeLibrary(h_pers);
sdr=-1;
}

void lir_sdr14_write(void *s, int bytes)
{
DWORD written;
P_Write(sdr14_handle,s,bytes, &written);
}

int lir_sdr14_read(void *s,int bytes)
{
DWORD i;
P_Read(sdr14_handle, s, bytes, &i);
return i;
}

void close_sdr14(void)
{
P_Close(sdr14_handle);
sdr=-1;
FreeLibrary(h_sdr14);
}


void open_sdr14(void)
{
sdrOpenEx P_OpenEx;
h_sdr14 = LoadLibrary("ftd2xx.dll");
if (h_sdr14 == NULL)
  {
  SNDLOG"Failed to load ftd2xx.dll\n");
  goto open_sdr14_fail2;
  }
SNDLOG"File ftd2xx.dll loaded.\n");
P_OpenEx=(sdrOpenEx)GetProcAddress(h_sdr14, "FT_OpenEx");
if(P_OpenEx == NULL)
  {
  SNDLOG"Could not find	FT_OpenEx\n");
  goto open_sdr14_fail;
  }
if(P_OpenEx(sdr14_name_string,
                         FT_OPEN_BY_DESCRIPTION, &sdr14_handle) != FT_OK)
  {
  SNDLOG"Could not open %s\n",sdr14_name_string);
  if(P_OpenEx(sdriq_name_string,
                          FT_OPEN_BY_DESCRIPTION, &sdr14_handle) != FT_OK)
    {
    SNDLOG"Could not open %s\n",sdriq_name_string);
    goto open_sdr14_fail;
    }
  else
    {
    SNDLOG"%s open OK\n",sdriq_name_string);
    }
  }
else
  {
  SNDLOG"%s open OK\n",sdr14_name_string);
  }
P_Read=(sdrRead)GetProcAddress(h_sdr14, "FT_Read");
P_Write=(sdrWrite)GetProcAddress(h_sdr14, "FT_Write");
P_Close=(sdrClose)GetProcAddress(h_sdr14, "FT_Close");
P_SetTimeouts=(sdrSetTimeouts)GetProcAddress(h_sdr14, "FT_SetTimeouts");
if(P_Read == NULL ||
   P_Write == NULL ||
   P_Close == NULL ||
   P_SetTimeouts == NULL)goto open_sdr14_fail;
if(P_SetTimeouts(sdr14_handle,500,500) != FT_OK)goto open_sdr14_fail;
sdr=1;
SNDLOG"open_sdr14() sucessful.\n");
return;
open_sdr14_fail:;
FreeLibrary(h_sdr14);
open_sdr14_fail2:;
sdr=-1;
SNDLOG"open_sdr14() returned an error.\n");
}

